from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GL.ARB.vertex_buffer_object import *
from OpenGL.raw.GL.constants import *
from vbo import VBO

def resize((width, height)):
	if height==0:
		height=1.0
	glViewport(0, 0, width, height)
	glMatrixMode(GL_PROJECTION)
	glLoadIdentity()
	gluPerspective(45, 1.0*width/height, 0.1, 100.0)
	glMatrixMode(GL_MODELVIEW)
	glLoadIdentity()


def drawcube(open):

	glBegin(GL_QUADS)

	if not open:
		# Front Face
		glNormal3f(0.0, 0.0, 1.0)
		glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -1.0,  1.0)
		glTexCoord2f(1.0, 0.0); glVertex3f( 1.0, -1.0,  1.0)
		glTexCoord2f(1.0, 1.0); glVertex3f( 1.0,  1.0,  1.0)
		glTexCoord2f(0.0, 1.0); glVertex3f(-1.0,  1.0,  1.0)
	
	# Back Face
	if not open:
		glNormal3f(0.0, 0.0, -1.0)
	else:
		glNormal3f(0.0, 0.0, 1.0)
	glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -1.0, -1.0)
	glTexCoord2f(0.0, 1.0); glVertex3f(-1.0,  1.0, -1.0)
	glTexCoord2f(1.0, 1.0); glVertex3f( 1.0,  1.0, -1.0)
	glTexCoord2f(1.0, 0.0); glVertex3f( 1.0, -1.0, -1.0)
	
	if not open:
		# Top Face
		glNormal3f(0.0, 1.0, 0.0)
		glTexCoord2f(1.0, 1.0); glVertex3f(-1.0,  1.0, -1.0)
		glTexCoord2f(1.0, 0.0); glVertex3f(-1.0,  1.0,  1.0)
		glTexCoord2f(0.0, 0.0); glVertex3f( 1.0,  1.0,  1.0)
		glTexCoord2f(0.0, 1.0); glVertex3f( 1.0,  1.0, -1.0)
	
	# Bottom Face
	if not open:
		glNormal3f(0.0, -1.0, 0.0)
	else:
		glNormal3f(0.0, 1.0, 0.0)
	glTexCoord2f(0.0, 1.0); glVertex3f(-1.0, -1.0, -1.0)
	glTexCoord2f(1.0, 1.0); glVertex3f( 1.0, -1.0, -1.0)
	glTexCoord2f(1.0, 0.0); glVertex3f( 1.0, -1.0,  1.0)
	glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -1.0,  1.0)
	
	# Right face
	if not open:
		glNormal3f(1.0, 0.0, 0.0)
	else:
		glNormal3f(-1.0, 0.0, 0.0)
	glTexCoord2f(0.0, 0.0); glVertex3f( 1.0, -1.0, -1.0)
	glTexCoord2f(0.0, 1.0); glVertex3f( 1.0,  1.0, -1.0)
	glTexCoord2f(1.0, 1.0); glVertex3f( 1.0,  1.0,  1.0)
	glTexCoord2f(1.0, 0.0); glVertex3f( 1.0, -1.0,  1.0)
	
	# Left Face
	if not open:
		glNormal3f(-1.0, 0.0, 0.0)
	else:
		glNormal3f(1.0, 0.0, 0.0)
	glTexCoord2f(1.0, 0.0); glVertex3f(-1.0, -1.0, -1.0)
	glTexCoord2f(0.0, 0.0); glVertex3f(-1.0, -1.0,  1.0)
	glTexCoord2f(0.0, 1.0); glVertex3f(-1.0,  1.0,  1.0)
	glTexCoord2f(1.0, 1.0); glVertex3f(-1.0,  1.0, -1.0)
	
	glEnd()


def initvbocube():
	vertices = [
		[-1.0,-1.0, 1.0], [ 1.0,-1.0, 1.0], [1.0, 1.0, 1.0], [-1.0, 1.0, 1.0], # Front
		[-1.0,-1.0,-1.0], [-1.0, 1.0,-1.0], [1.0, 1.0,-1.0], [ 1.0,-1.0,-1.0], # Back
		[-1.0, 1.0,-1.0], [-1.0, 1.0, 1.0], [1.0, 1.0, 1.0], [ 1.0, 1.0,-1.0], # Top
		[-1.0,-1.0,-1.0], [ 1.0,-1.0,-1.0], [1.0,-1.0, 1.0], [-1.0,-1.0, 1.0], # Bottom
		[ 1.0,-1.0,-1.0], [ 1.0, 1.0,-1.0], [1.0, 1.0, 1.0], [ 1.0,-1.0, 1.0], # Right
		[-1.0,-1.0,-1.0], [-1.0,-1.0, 1.0],[-1.0, 1.0, 1.0], [-1.0, 1.0,-1.0]  # Left
	]

	texcoords = [
		[1.0, 1.0], [0.0, 1.0], [0.0, 0.0], [1.0, 0.0], # Front
		[0.0, 1.0], [0.0, 0.0], [1.0, 0.0], [1.0, 1.0], # Back
		[1.0, 0.0], [1.0, 1.0], [0.0, 1.0], [0.0, 0.0], # Top
		[0.0, 0.0], [1.0, 0.0], [1.0, 1.0], [0.0, 1.0], # Bottom
		[0.0, 1.0], [0.0, 0.0], [1.0, 0.0], [1.0, 1.0], # Right
		[1.0, 1.0], [0.0, 1.0], [0.0, 0.0], [1.0, 0.0]  # Left
	]


	normals = [
		 [0.0, 0.0, 1.0], [0.0, 0.0, 1.0], [ 0.0, 0.0, 1.0], [ 0.0, 0.0, 1.0], # Front
		 [0.0, 0.0,-1.0], [0.0, 0.0,-1.0], [ 0.0, 0.0,-1.0], [ 0.0, 0.0,-1.0], # Back
		 [0.0, 1.0, 0.0], [0.0, 1.0, 0.0], [ 0.0, 1.0, 0.0], [ 0.0, 1.0, 0.0], # Top
		 [0.0,-1.0, 0.0], [0.0,-1.0, 0.0], [ 0.0,-1.0, 0.0], [ 0.0,-1.0, 0.0], # Bottom
		 [1.0, 0.0, 0.0], [1.0, 0.0, 0.0], [ 1.0, 0.0, 0.0], [ 1.0, 0.0, 0.0], # Right
		[-1.0, 0.0, 0.0],[-1.0, 0.0, 0.0], [-1.0, 0.0, 0.0], [-1.0, 0.0, 0.0]  # Left
	]

	vbo = VBO()
	vbo.set_vertices(vertices, GL_STATIC_DRAW)
	vbo.set_texcoords(texcoords, GL_STATIC_DRAW)
	vbo.set_normals(normals, GL_STATIC_DRAW)

	return vbo


def drawquad():
	glBegin(GL_QUADS)
	glMultiTexCoord2f(GL_TEXTURE0, 0.0, 0.0)
	glMultiTexCoord2f(GL_TEXTURE1, 0.0, 0.0)
	glMultiTexCoord2f(GL_TEXTURE2, 0.0, 0.0)
	glVertex2f(-1.0, -1.0)
	glMultiTexCoord2f(GL_TEXTURE0, 1.0, 0.0)
	glMultiTexCoord2f(GL_TEXTURE1, 1.0, 0.0)
	glMultiTexCoord2f(GL_TEXTURE2, 1.0, 0.0)
	glVertex2f(1.0, -1.0)
	glMultiTexCoord2f(GL_TEXTURE0, 1.0, 1.0)
	glMultiTexCoord2f(GL_TEXTURE1, 1.0, 1.0)
	glMultiTexCoord2f(GL_TEXTURE2, 1.0, 1.0)
	glVertex2f(1.0, 1.0)
	glMultiTexCoord2f(GL_TEXTURE0, 0.0, 1.0)
	glMultiTexCoord2f(GL_TEXTURE1, 0.0, 1.0)
	glMultiTexCoord2f(GL_TEXTURE2, 0.0, 1.0)
	glVertex2f(-1.0, 1.0)
	glEnd()
