from OpenGL.GL import *

class Light(object):
	def __init__(self, num):
		self.__max_lights = glGetInteger(GL_MAX_LIGHTS) # FIXME: move in a glContext class
		
		if num > self.__max_lights:
			raise Error

		self.__id = getattr(OpenGL.GL, 'GL_LIGHT%d' % num)
		self.__ambient = (0.0, 0.0, 0.0, 1.0)
		self.__diffuse = (1.0, 1.0, 1.0, 1.0)
		self.__specular = (1.0, 1.0, 1.0, 1.0)
		self.__position = (0.0, 0.0, 0.0, 1.0)
		self.__const_att = 0.2 # 1.5
		self.__lin_att = 0.1 # 0.5
		self.__quad_att = 0.01 # 0.25

		self.update()

		glEnable(self.__id)

	def __del__(self):
		glDisable(self.__id)

	
	def __get_id(self):
		return self.__id


	def __set_id(self, num):
		if num > self.__max_lights:
			raise Error

		self.__id = getattr(OpenGL.GL, 'GL_LIGHT%d' % num)

	id = property(__get_id, __set_id)


	def __get_ambient(self):
		return self.__ambient

	def __set_ambient(self, ambient):
		if len(ambient) == 4:
			self.__ambient = ambient
		elif len(ambient) == 3:
			self.__ambient = ambient + (1.0, )
		glLightfv(self.__id, GL_AMBIENT, self.__ambient)

	ambient = property(__get_ambient, __set_ambient)


	def __get_diffuse(self):
		return self.__diffuse
			        
	def __set_diffuse(self, diffuse):
		if len(diffuse) == 4:
			self.__diffuse = diffuse
		elif len(diffuse) == 3:
			self.__diffuse = diffuse + (1.0, )
		glLightfv(self.__id, GL_DIFFUSE, self.__diffuse)

	diffuse = property(__get_diffuse, __set_diffuse)


	def __get_specular(self):
		return self.__specular

	def __set_specular(self, specular):
		if len(specular) == 4:
			self.__specular = specular
		elif len(specular) == 3:
			self.__specular = specular + (1.0, )
		glLightfv(self.__id, GL_SPECULAR, self.__specular)

	specular = property(__get_specular, __set_specular)


	def __get_position(self):
		return self.__position
			        
	def __set_position(self, position):
		if len(position) == 4:
			self.__position = position
		elif len(position) == 3:
			self.__position = position + (1.0, ) # default to point lights
		glLightfv(self.__id, GL_POSITION, self.__position)

	position = property(__get_position, __set_position)


	def __get_const_att(self):
		return self.__const_att

	def __set_const_att(self, const_att):
		self.__const_att = const_att
		glLightf(self.__id, GL_CONSTANT_ATTENUATION, self.__const_att)
	
	const_att = property(__get_const_att, __set_const_att)

	def __get_lin_att(self):
		return self.__lin_att

	def __set_lin_att(self, lin_att):
		self.__lin_att = lin_att
		glLightf(self.__id, GL_LINEAR_ATTENUATION, self.__lin_att)

	lin_att = property(__get_lin_att, __set_lin_att)

	def __get_quad_att(self):
		return self.__lin_att

	def __set_quad_att(self, quad_att):
		self.__quad_att = quad_att
		glLightf(self.__id, GL_QUADRATIC_ATTENUATION, self.__quad_att)

	const_att = property(__get_lin_att, __set_lin_att)


	def enable(self):
		glEnable(self.__id)

	def disable(self):
		glDisable(self.__id)


	def update(self):
		glLightfv(self.__id, GL_AMBIENT, self.__ambient)
		glLightfv(self.__id, GL_DIFFUSE, self.__diffuse)
		glLightfv(self.__id, GL_POSITION, self.__position)
		glLightf(self.__id, GL_CONSTANT_ATTENUATION, self.__const_att)
		glLightf(self.__id, GL_LINEAR_ATTENUATION, self.__lin_att)
		glLightf(self.__id, GL_QUADRATIC_ATTENUATION, self.__quad_att)


	def draw(self):
		pos = self.position

		glPushAttrib(GL_ENABLE_BIT | GL_CURRENT_BIT)
		glDisable(GL_TEXTURE_2D)
		glDisable(GL_LIGHTING)

		glColor4fv(self.__diffuse)
		glPushMatrix()
		glTranslate(pos[0], pos[1], pos[2])
		glBegin(GL_QUADS)
		glVertex3f( 0.025, 0.025, 0)
		glVertex3f(-0.025, 0.025, 0)
		glVertex3f(-0.025,-0.025, 0)
		glVertex3f( 0.025,-0.025, 0)
		glEnd()
		glPopMatrix()

		glPopAttrib()
