#!/usr/bin/env python

import os
from OpenGL.GL import *
from OpenGL.GL.EXT.framebuffer_object import *
import pygame, pygame.image
from pygame.locals import *

from fbo import FBO
from texture import Texture
from shader import Shader
from light import Light
from material import Material
from timer import Timer
from functions import drawcube, resize, initvbocube, drawquad

from math import sin, cos
from optparse import OptionParser

SHADERS_DIR = os.path.join('shaders', 'multilight')
NUM_LIGHTS = 3
tex0 = tex1 = None
lights = []
bg = True
movz = 0.0; roty = 0.0


def main():
	global movz, roty, tex0, tex1, lights

	interval = 0.0
	movl = 0.0
	movelights = False
	move = False; rotate = True
	rendermode = 0
	mpsh = None
#	withVBO = False

# Parser class
	parser = OptionParser()
	parser = OptionParser(usage="usage: %prog [options]", version="%prog 0.1")
	parser.add_option("-x", "--width", type="int", dest="width", default=640,
					help="To set the window width (default: %default)")
	parser.add_option("-y", "--height", type="int", dest="height", default=480,
					help="To set the window height (default: %default)")
	parser.add_option("-f", "--fullscreen", action="store_true", dest="fullscreen", default=False,
					help="To set fullscreen mode (default: %default)")
	(options, args) = parser.parse_args()

# PyGame init
	video_flags = OPENGL|DOUBLEBUF

	if options.fullscreen:
		video_flags |= FULLSCREEN
    
	pygame.init()
	pygame.display.set_caption('PyGLSL_multilight by Encelo')
	surface = pygame.display.set_mode((options.width, options.height), video_flags)

# OpenGL basic init
	glShadeModel(GL_SMOOTH)
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST)
	glClearColor(0.0, 0.0, 0.0, 1.0)
	glClearDepth(1.0)
	glEnable(GL_DEPTH_TEST)
	glDepthFunc(GL_LEQUAL) # Either set this or clear the Z-buffer at each scene rendering pass
	glBlendFunc(GL_ONE, GL_ONE)
	glEnable(GL_TEXTURE_2D)
	glEnable(GL_RESCALE_NORMAL) # Avoid improper lighting of the uniformly scaled brick room
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, (0.2, 0.2, 0.2, 1.0))
	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE)
	resize((640,480))

# OpenGL classes init
#	vbo = initvbocube()	

	fbo = FBO(options.width, options.height)
	fbo.attach_render(GL_DEPTH_COMPONENT24)
	for i in range(3):
		fbo.attach_texture(GL_RGBA16F_ARB)
	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0)
	glDrawBuffer(GL_BACK)

	pv1sh = Shader(os.path.join(SHADERS_DIR, 'pervertex1.vert'),     os.path.join(SHADERS_DIR, 'pervertex1.frag'))
	pp1sh = Shader(os.path.join(SHADERS_DIR, 'perpixel1.vert'),      os.path.join(SHADERS_DIR, 'perpixel1.frag'))
	pv3sh = Shader(os.path.join(SHADERS_DIR, 'pervertex3.vert'),     os.path.join(SHADERS_DIR, 'pervertex3.frag'))
	pp3sh = Shader(os.path.join(SHADERS_DIR, 'perpixel3.vert'),      os.path.join(SHADERS_DIR, 'perpixel3.frag'))
	def1st= Shader(os.path.join(SHADERS_DIR, 'deferredfirst.vert'),  os.path.join(SHADERS_DIR, 'deferredfirst.frag'))
	def2nd= Shader(os.path.join(SHADERS_DIR, 'deferredsecond.vert'), os.path.join(SHADERS_DIR, 'deferredsecond.frag'))
	def2nd.update_uniform('Tex0', 0)
	def2nd.update_uniform('Tex1', 1)
	def2nd.update_uniform('Tex2', 2)
	glUseProgram(0)

	tex0 = Texture(os.path.join('textures', 'tex0.jpg'))
	tex1 = Texture(os.path.join('textures', 'tex1.png'))

	lights.append(Light(0))
	lights[0].position = (0, 2, -2)
	lights[0].diffuse = (1.0, 0.0, 0.0)
	lights.append(Light(1))
	lights[1].position = (-5.5, 2, -2)
	lights[1].diffuse = (0.0, 1.0, 0.0)
	lights.append(Light(2))
	lights[2].position = (5.5, 2, -2)
	lights[2].diffuse = (0.0, 0.0, 1.0)

	mat = Material()

	timer = Timer()

# Event Cycle
	while 1:
		event = pygame.event.poll()
		if event.type == QUIT:
			break
		if event.type == KEYDOWN:
			if event.key == K_ESCAPE:
				return 0

			elif event.key == K_m:
				movelights = not movelights
			elif event.key == K_SPACE:
				move = not move
			elif event.key == K_r:
				rotate = not rotate
			elif event.key == K_b:
				bg = not bg
			elif event.key == K_0:
				movz = 0.0
				roty = 0.0
				movl = 0.0
			elif event.key == K_1:
				glUseProgram(0)
				glDisable(GL_LIGHTING)
				rendermode = 0
				print('No lighting at all')
				timer.reset()
			elif event.key == K_2:
				glUseProgram(0)
				glEnable(GL_LIGHTING)
				rendermode = 0
				print('Per-vertex single-pass fixed pipeline lighting')
				timer.reset()
			elif event.key == K_3:
				glUseProgram(0)
				glEnable(GL_LIGHTING)
				mpsh = None
				rendermode = 1
				print('Per-vertex multi-pass fixed pipeline lighting')
			elif event.key == K_4:
				mpsh = pv1sh
				rendermode = 1
				print('Per-vertex multi-pass shader lighting')
				timer.reset()
			elif event.key == K_5:
				mpsh = pp1sh
				rendermode = 1
				print('Per-pixel multi-pass shader lighting')
				timer.reset()
			elif event.key == K_6:
				pv3sh.use()
				rendermode = 0
				print('Per-vertex single-pass shader lighting')
				timer.reset()
			elif event.key == K_7:
				pp3sh.use()
				rendermode = 0
				print('Per-pixel single-pass shader lighting')
				timer.reset()
#			elif event.key == K_8:
#				rendermode = 2
#				print('Deferred shading lighting')
#				timer.reset()
#			elif event.key == K_v:
#				withVBO = not withVBO


# Scene rendering
		if move:
			movz += 0.0025 * interval
		if rotate:
			roty += 0.1 * interval
		if movelights:
			movl += 0.0025 * interval;

		pos = lights[0].position
		lights[0].position = (2 * sin(movl), pos[1], -5 + cos(movl) * 5.0)
		pos = lights[1].position
		lights[1].position = (pos[0], 2 + sin(movl) * 2.0, -5 + cos(movl) * 5.0)
		pos = lights[2].position
		lights[2].position = (pos[0], 2 + sin(movl) * 2.0, -5 + cos(movl) * 5.0)

		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
		glLoadIdentity()
		glTranslatef(0.0, -2.0, -10.0)
		glRotatef(15.0, 1.0, 0.0, 0.0)

		if rendermode == 0: # Single-pass
			# Restore from GL_LIGHT0 "recycling"
			for i in range(NUM_LIGHTS):
				lights[i].id = i
			lights[0].update()
			drawscene()
		elif rendermode == 1: # Multi-pass
			if mpsh == None:
				for i in range(1, NUM_LIGHTS):
					glDisable(GL_LIGHT0+i)

			if (mpsh):
				mpsh.use()
			for i in range(NUM_LIGHTS):
				if i > 0:
					glLightModelfv(GL_LIGHT_MODEL_AMBIENT, (0.0, 0.0, 0.0, 1.0))
					glEnable(GL_BLEND)
				# "Recycling" of GL_LIGHT0
				lights[i].id = 0
				lights[i].update()
				drawscene()

			glDisable(GL_BLEND)
			glLightModelfv(GL_LIGHT_MODEL_AMBIENT, (0.2, 0.2, 0.2, 1.0))

			if mpsh == None:
				for i in range(1, NUM_LIGHTS):
					glEnable(GL_LIGHT0+i)
		elif rendermode == 2: # Deferred lighting
			# First pass: scene rendering
			fbo.bind_in()
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
			def1st.use()
			drawscene()

			# Second pass: deferred lighting
			fbo.bind_out()
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
			def2nd.use()

			glDisable(GL_DEPTH_TEST) # Prevent the z-buffer from being updated with quad depth values
			glMatrixMode(GL_PROJECTION)
			glPushMatrix()
			glLoadIdentity()
			glMatrixMode(GL_MODELVIEW)

			for i in range(NUM_LIGHTS):
				if i > 0:
					glLightModelfv(GL_LIGHT_MODEL_AMBIENT, (0.0, 0.0, 0.0, 1.0))
					glEnable(GL_BLEND)
				# "Recycling" of GL_LIGHT0
				lights[i].id = 0
				lights[i].update() # Setting light position is influenced by the current MODELVIEW matrix
				glPushMatrix()
				glLoadIdentity()
				drawquad()
				glPopMatrix()

			glDisable(GL_BLEND)
			glLightModelfv(GL_LIGHT_MODEL_AMBIENT, (0.2, 0.2, 0.2, 1.0))

			glMatrixMode(GL_PROJECTION)
			glPopMatrix()
			glMatrixMode(GL_MODELVIEW)
			glEnable(GL_DEPTH_TEST)
			
			glActiveTexture(GL_TEXTURE0)

		# Drawing lights in traditional forward rendering, on top of everythign else, no matter current rendering mode
		drawlights()

		pygame.display.flip()
		interval = timer.addframe()


def drawscene():
	global movz, roty, tex0, tex1

	if bg:
		tex0.bind()
		glPushMatrix()
		glTranslatef(0.0, 6.0, -10.0)
		glScalef(10.0, 10.0, 10.0)
		drawcube(True)
		glPopMatrix()

	tex1.bind()
	for i in range(5):
		glPushMatrix()
		glTranslatef(0.0, 0.0, 3.0 * sin(movz))
		glTranslatef(-2.0, 0.0, 0.0)
		glTranslatef(i * 2.0, 0.0, i * (-4.0))
		glRotatef(roty, 0.0, 1.0, 0.0)
		drawcube(False)
		glPopMatrix()


def drawlights():
	global lights

	glPushAttrib(GL_ENABLE_BIT | GL_CURRENT_BIT)
	glDisable(GL_TEXTURE_2D)
	glDisable(GL_LIGHTING)
#	prog = glGetInteger(GL_CURRENT_PROGRAM) #XXX: Unknown specifier error
#	glUseProgram(0)
	for i in range(NUM_LIGHTS):
		lights[i].draw()
	glPopAttrib()
#	glUseProgram(prog)


if __name__ == '__main__':
	main()
